// ***********************************************************
// Persistence Of Vision Ray Tracer Scene Description File
// File name  : boltstill.pov
// Version    : 3.5
// Description: bolts & nuts
// Date       : Mar-Apr 2002, May-2003: adapted for LightsysIV
// Author     : Jaime Vives Piqueres
// Scene scale: 1 pov unit = 1 cm
// ***********************************************************

// >= +w640 +h480

#include "colors.inc"
#include "textures.inc"
#include "metals.inc"

// *** control center ***
#declare use_bolts  =1;  // place the bolts?
#declare use_area   =0;  // use area lights?
#declare use_blur  =50;  // blur samples (0=off)
#declare use_norm   =1;  // use micro-normals?
#declare hf_res  =1000;  // hf resolution
#declare r_l=seed(132);  // random light placement seed 

global_settings{
  ambient_light 0.0
  assumed_gamma 1.0
}


// ********************
// *** CIE+lightsys ***
// ********************
#include "CIE.inc"
#declare ColSys=sRGB_ColSys;
//CIE_ColorSystemWhitepoint(ColSys,Illuminant_A)
CIE_ColorSystemWhitepoint(ColSys,Blackbody2Whitepoint(4000))
//CIE_ColorSystemWhitepoint(ColSys,Daylight2Whitepoint(5500))
#include "rspd_jvp.inc" // material samples

#include "lightsys.inc"
#include "lightsys_constants.inc"
#include "lightsys_colors.inc"
#declare Lightsys_Brightness=6; 


// *******************
// *** build lamps ***
// *******************
#declare fl_lm=2000;
// + lamp object
#macro sheet(cl)
box{-.5,.5
 material{
  texture{
   pigment { color rgbf<1, 1, 1, 1> }
   finish {  diffuse 0 }
  }
  interior{
   media {
    method 1
    emission cl
    intervals 10
    samples 1, 10
    confidence 0.9999
    variance 1/1000
   }
  }
 }
 hollow
 no_shadow
}
#end
#macro bulb(cl)
sphere{0,.5
 material{
  texture{
   pigment { color rgbf<1, 1, 1, 1> }
   finish {  diffuse 0 }
  }
  interior{
   media {
    method 1
    emission cl
    intervals 10
    samples 1, 10
    confidence 0.9999
    variance 1/1000
   }
  }
 }
 hollow
 no_shadow
}
#end

// + build the lamps 
// try changin the r_l seed to find interesting lighting situations
union{
 Light(Cl_Incandescent_60w,400,<9,0,0>,<0,0,9>,4*use_area,4*use_area,1)
 object{bulb(Cl_Incandescent_60w)scale <2,3,2>}
 translate (-200+400*rand(r_l))*z
 translate (250*rand(r_l))*y
 rotate 360*rand(r_l)*y
}
#if (rand(r_l)>.5)
union{
 Light(Cl_Cool_White_Fluor,900,<48,0,0>,<0,0,48>,6*use_area,6*use_area,1)
 object{sheet(Cl_Cool_White_Fluor*900)scale <24,.1,3> translate .9*y}
 translate (-200+400*rand(r_l))*z
 translate (-200+400*rand(r_l))*x
 translate (250*rand(r_l))*y
}
#else
union{
 Light(Cl_SI_D65,3000,<100,0,0>,<0,0,100>,6*use_area,6*use_area,1)
 object{sheet(Cl_SI_D65*2000)scale <100,.1,100> translate .9*y}
 rotate -90*x
 translate (-249)*z
 translate (250*rand(r_l))*y
 rotate 360*rand(r_l)*y
}
#end


// *************************
// *** OLD BRUSHED METAL ***
// *************************
// + microsurface normal
#declare p_micro2=
pigment{
 crackle turbulence 1
 color_map{
  [0.00 White*.7]
  [0.01 White*.7]
  [0.02 White]
  [1.00 White]
 }
 scale 5
}
#declare p_micro1=
pigment{
 cells 
 turbulence .5
 color_map{
  [0.0 White*.9]
  [1.0 White]
 }
 scale .01
}
// + average all the normals
#declare p_brushed_new=
pigment{
 average turbulence 0
 pigment_map{
  [0.0 p_micro1]
  [0.1 p_micro1 rotate 45*x]
  [0.2 p_micro1 rotate -45*x]
  [0.3 p_micro2]
  [0.4 p_micro1 rotate 45*y]
  [0.5 p_micro1 rotate -45*y]
  [0.6 p_micro1 rotate 90*z]
  [0.7 p_micro2 rotate 45]
  [0.8 p_micro1 rotate 45*z]
  [0.9 p_micro1 rotate 90*y]
  [1.0 p_micro1 rotate -45*z]
 }
 scale .1
}

// + declare the final normal as image_pattern
#declare n_brushed_new=
normal{pigment_pattern{p_brushed_new} .4}
// + declare the final metal texture
#declare t_metal=
texture{
 pigment{
//  rgb ReflectiveSpectrum(RS_BoltMetal2)
//  rgb ReflectiveSpectrum(RS_Nickel1)
//  rgb ReflectiveSpectrum(RS_NickelBrass1)
  rgb ReflectiveSpectrum(RS_NordicGold1)
//  rgb ReflectiveSpectrum(RS_Iron)
//  rgb ReflectiveSpectrum(RS_StainlessSteel1)
 }
 finish{
  Metal
 }
 #if (use_norm)
 normal{n_brushed_new .5}
 #end
}


// ********************
// *** BOLTS & NUTS ***
// ********************
#if (use_bolts)
#declare bolt=
union{
 intersection{
  union{
   cylinder{<0,-.3,0>,<0,.3,0>,.6}
   torus{.6,.3}
  }
  plane{z,.72}
  plane{z,-.72 inverse}
  plane{x,0 rotate 30*y inverse translate -.82*x}
  plane{x,0 rotate -30*y inverse translate -.82*x}
  plane{x,0 rotate 30*y translate .82*x}
  plane{x,0 rotate -30*y translate .82*x}
 }
 height_field{
  png "hf_bolt"
  smooth
  water_level 0.1
  translate -.5
  scale <1.3,.1,1.3>
  translate <0,.27,0>
 }
 difference{
  cylinder{<0,-.3,0>,<0,-3,0>,.4}
  #declare rs=.08;
  #declare ns=14;
  #declare is=1;
  #while (is<=ns)
   torus{.4,rs scale <1,.8,1> rotate -5*z translate -(.4+(rs*2+.01)*is)*y}
   #declare is=is+1;
  #end
  torus{.45,rs*2 translate -(.4+(rs*2+.01)*(ns+1.1))*y}
  sphere{0,.3 scale <1,.25,1> translate -3*y}
  sphere{0,.2 scale <1,.5,1> translate -2.97*y}
 }
 cylinder{<0,-.3,0>,<0,-2.8,0>,.31}
}
#declare nut=
difference{
 intersection{
  difference{
   union{
    cylinder{<0,-.3,0>,<0,.3,0>,.6}
    torus{.6,.3}
   }
   height_field{
    png "hf_nut"
    translate -.5
    rotate 180*x
    scale <1.3,.2,1.3>
    translate <0,.31,0>
   }
  }
  plane{z,.72}
  plane{z,-.72 inverse}
  plane{x,0 rotate 30*y inverse translate -.82*x}
  plane{x,0 rotate -30*y inverse translate -.82*x}
  plane{x,0 rotate 30*y translate .82*x}
  plane{x,0 rotate -30*y translate .82*x}
  plane{y,-.29 inverse}
 }
 cylinder{<0,-.4,0>,<0,.4,0>,.4}
 torus{.4,.05}
 torus{.4,.05 rotate 2*y translate .10*y}
 torus{.4,.05 rotate 2*y translate -.10*y}
 torus{.4,.05 rotate 2*y translate .20*y}
 torus{.4,.05 rotate 2*y translate -.20*y}
 torus{.4,.05 rotate 2*y translate .30*y}
 torus{.4,.05 rotate 2*y translate -.30*y}
}
#declare arandela=
union{
 torus{1,.02 translate .03*y}
 difference{
  cylinder{<0,-.05,0>,<0,.05,0>,1}
  cylinder{<0,-.06,0>,<0,.06,0>,.7}
 }
 difference{
  cylinder{<0,-.03,0>,<0,.03,0>,1.02}
  cylinder{<0,-.04,0>,<0,.04,0>,1}
 }
 torus{1,.02 translate -.03*y}
}

#declare rn=seed(373); // radom texturing
union{
 object{nut rotate 360*rand(rn)*y
  texture{t_metal translate 1000*rand(rn)}
  translate 80.31*y
 }
 object{nut rotate 360*rand(rn)*y
  texture{t_metal translate 1000*rand(rn)}
  translate <-.7,80.31,-1.7>
 }
 object{nut rotate 360*rand(rn)*y
  texture{t_metal translate 1000*rand(rn)}
  translate <1.7,80.31,2>
 }
 object{bolt
  rotate <83.7,45,0>
  texture{t_metal translate 1000*rand(rn)}
  translate <2,80.7,-2>
 }
 object{bolt
  rotate 180*x
  texture{t_metal translate 1000*rand(rn)}
  translate <-2,80.33,1>
 }
 object{arandela
  texture{t_metal translate 1000*rand(rn)}
  rotate <-11,62,0>
  translate <-2.2,80.6,1>
 }
 object{bolt
  rotate <83.7,165,0>
  texture{t_metal translate 1000*rand(rn)}
  translate <-2.5,80.7,-3.5>
 }
 object{bolt
  rotate <83.7,90,0>
  texture{t_metal translate 1000*rand(rn)}
  translate <.5,80.7,4.5>
 }
 object{bolt
  rotate 180*x
  texture{t_metal translate 1000*rand(rn)}
  translate <2.9,80.33,4.5>
 }
 object{arandela
  texture{t_metal translate 1000*rand(rn)}
  translate <4.6,80.05,3.8>
 }
 object{nut rotate 360*rand(rn)*y
  texture{t_metal translate 1000*rand(rn)}
  translate <-3.5,80.31,6>
 }
}
#end


// *****************
// *** test room ***
// *****************
#declare p_mortar=rgb ReflectiveSpectrum(RS_ConstrStone2);
#declare p_brick =rgb ReflectiveSpectrum(RS_ConstrStone3);
box{-.5,.5
 hollow
 scale <500,250,500>
 pigment{brick 
  color p_mortar color p_brick
  scale 4
 }
 translate 125*y
}
plane{y,0.1
 hollow
 pigment{checker color p_mortar color p_mortar*.1 scale 50}
}
plane{y,249.9
 hollow
 pigment{rgb ReflectiveSpectrum(RS_White_Paint_1)}
}

// *** table ***
// - pigment for the table hf function -
#declare p_hf=
pigment{
 brick color 1 color rgb 0 mortar 2 rotate 90*x scale .001
}
// - height filed table -
#declare table=
height_field{
 function hf_res,hf_res{
  pigment{p_hf}
 }
 water_level 0.1
 translate -.5
 texture{
  pigment{rgb ReflectiveSpectrum(RS_Iron)}
  finish{Metal}
 }
 scale <50,.05,50>
}
object{table
 translate <0,80,2>
}


// **************
// *** camera ***
// **************
#declare cl=<0,91,-18>;
#declare la=<0,81.1,0>;
#declare r_c=seed(19);
camera{
 location cl
 up 2.4*y
 right 3.2*x
 direction 5*z
 look_at la
 #if (use_blur)
 focal_point la
 aperture 1
 blur_samples use_blur
 variance 0
 #end
 rotate 360*rand(r_c)*y
}

